//******************************************************************************
// AWS File Browser - A browser for files containing  AWS tape format data.
// By Rob Storey December 2001 - intabits@optushome.com.au
// This program and its source code are in the public domain.
// Anyone is free to do whatever they want with it,
// except blame me when whatever they do with it screws something up.
//******************************************************************************
Unit AWSMain;

Interface

Uses Windows, Messages, SysUtils, Classes, Graphics,
     Controls, Forms, Dialogs, StdCtrls, ExtCtrls, Menus,
     AWSAbout;

Type TAWSHeader=Record
       ThisSize: Word;
       PrevSize: Word;
       Flags:    Byte;
       Rsvd:     Byte;
     End;

Const Heading='AWS File Browser';
      MaxRecSize=32768;
      AWSHdrSize=SizeOf(TAWSHeader);
      AFBegRec=$80;
      AFTapeMK=$40;
      AFEndRec=$20;
      AFFulrec=AFBegRec+AFEndRec;

Type
  TMainForm = class(TForm)
    LBLog:      TListBox;
    Splitter:   TSplitter;
    LBDump:     TListBox;
    OpenDialog: TOpenDialog;
    MainMenu: TMainMenu;
    MIOpen: TMenuItem;
    MIExit: TMenuItem;
    MIHelp: TMenuItem;
    procedure MIExitClick(Sender: TObject);
    procedure MIOpenClick(Sender: TObject);
    procedure LBLogClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure MIHelpClick(Sender: TObject);
  private
    EXEPath:   String;
    AWSPath:   String;
    AWSFile:   File;
    AWSOpen:   Boolean;
    AWSPosn:   Longint;
    AWSHeader: TAWSHeader;
    RecBuffer: Array[0..MaxRecSize-1] of Byte;
    Procedure Log(LS: String);
    Procedure DumpRecord(RP: Longint);
    Procedure ReadChunk;
    Function  GenDumpLine(Var RO: Word; RS: Word): String;
  public
  end;

Var MainForm: TMainForm;

Implementation

{$R *.DFM}

// EBCDIC to ASCII conversion table
Const CharMap: String=
//     0000000000000000111111111111111122222222222222223333333333333333
//     0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF
      '................................................................'+
//     4444444444444444555555555555555566666666666666667777777777777777
//     0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF
      ' ...........<(+|&.........!$*).^-/..........%_>?...........#@.="'+
//     88888888888888889999999999999999AAAAAAAAAAAAAAAABBBBBBBBBBBBBBBB
//     0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF
      '.abcdefghi.{.(+..jklmnopqr.}.)....stuvwxyz......................'+
//     CCCCCCCCCCCCCCCCDDDDDDDDDDDDDDDDEEEEEEEEEEEEEEEEFFFFFFFFFFFFFFFF
//     0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF
      '.ABCDEFGHI.......JKLMNOPQR........STUVWXYZ......0123456789......';

procedure TMainForm.FormCreate(Sender: TObject);
Var CF: TextFile;
begin
  // Set heading, Enable hint display
  Caption:=Heading;
  Application.ShowHint:=True;
  // Note where we are running from
  EXEPath:=ExtractFilePath(Application.EXEName);
  // Load last path selected
  If FileExists(EXEPath+'AWSBrows.CTL') then begin
    AssignFile(CF,'AWSBrows.CTL'); Reset(CF);
    Readln(CF,AWSPath);
    CloseFile(CF);
  End;
end;

procedure TMainForm.MIHelpClick(Sender: TObject);
begin
  AboutForm.Show;
end;

procedure TMainForm.MIExitClick(Sender: TObject);
begin
  // Close file if open, Close app.
  If AWSOpen then CloseFile(AWSFile);
  Close;
end;

procedure TMainForm.MIOpenClick(Sender: TObject);
Var CF: TextFile;
    PN: String;
begin
  // Select file (from last path opened)
  With OpenDialog do begin
    InitialDir:=AWSPath;
    If Not Execute then Exit
                   else PN:=FileName;
  End;
  Screen.Cursor:=CRHourGlass;
  // Save path opened in control file
  AWSPath:=ExtractFilePath(PN);
  AssignFile(CF,EXEPath+'AWSBrows.CTL'); Rewrite(CF);
  Writeln(CF,AWSPath);
  CloseFile(CF);
  // Set file name in caption
  Caption:=Heading+' - '+PN;
  // Clear record list and dump displays
  LBLog.Clear; LBDump.Clear;
  Application.ProcessMessages;
  // Open file and process chunks
  Try
    AssignFile(AWSFile,PN); FileMode:=0; Reset(AWSFile,1);
    AWSOpen:=True;
    AWSPosn:=0;
    While Not Eof(AWSFile) do ReadChunk;
  Except
    On E: Exception do begin
      Log('Error: '+E.Message);
      AWSOpen:=False;
    End;
  End;
  Screen.Cursor:=CRDefault;
End;

procedure TMainForm.LBLogClick(Sender: TObject);
Var RO: Longint;
begin
  // Get file offset that's stored with record info
  With LBLog,Items do
    RO:=Longint(Objects[ItemIndex]);
  // Dump the record
  DumpRecord(RO);
end;

procedure TMainForm.Log(LS: String);
begin
  // Add record info to list box, also display and store its offset
  LBLog.Items.AddObject(Format('%9D %S',[AWSPosn,LS]),TObject(AWSPosn));
end;

procedure TMainForm.ReadChunk;
begin
  // Note chunk offset, Read its header
  AWSPosn:=FilePos(AWSFile);
  BlockRead(AWSFile,AWSHeader,AWSHdrSize);
  // Check out header flags
  With AWSHeader do begin
    Case Flags of
      AFTapeMK: Log('----------TM-----------');
      AFFulRec: Begin
                  Log('Full Record. Size='+IntToStr(ThisSize));
                  Seek(AWSFile,AWSPosn+AWSHdrSize+ThisSize);
                End;
      Else      Raise Exception.Create('Unknown flag code: $'+IntToHex(Flags,1));
    End;
  End;
End;

procedure TMainForm.DumpRecord(RP: Integer);
Var RS,RO: Word;
begin
  // Clear dump display
  LBDump.Clear;
  // Move to start of chunk, Read header
  Seek(AWSFile,RP);
  BlockRead(AWSFile,AWSHeader,AWSHdrSize);
  // Get size of chunk data, Ignore if zero or wrong record type
  With AWSHeader do begin
    If Flags<>AFFulRec then Exit;
    RS:=ThisSize;
    If RS=0 then Exit;
  End;
  // Clear record buffer, Read new record data
  FillChar(RecBuffer,MaxRecSize,#0);
  BlockRead(AWSFile,RecBuffer,RS);
  // Dump 16 byte lines until end of record
  RO:=0;
  Repeat
    LBDump.Items.Add(GenDumpLine(RO,RS));
  Until RO>=RS;
end;

Function TMainForm.GenDumpLine(Var RO: Word; RS: Word): String;
Var BI,BB: Byte;
    SH,SC: String;
Begin
  // Init hex part as record offset, Clear char part
  SH:=IntToHex(RO,4); SC:='';
  // Process 16 bytes: Form hex, space out words, form char part
  For BI:=1 to 16 do begin
    If (BI Mod 4)=1 then SH:=SH+' ';
    BB:=RecBuffer[RO]; Inc(RO);
    SH:=SH+IntToHex(BB,2);
    SC:=SC+CharMap[BB+1];
    If RO>=RS then Break;
  End;
  // Pad short hex part (last line)
  While Length(SH)<40 do SH:=SH+' ';
  // Return hex and char parts. (And updated offset)
  Result:=SH+'  '+SC;
End;

End.


